@@ -492,7 +492,7 @@ static void extract_generic_string(struct obj_file *f, MODULE *mod)
 		while (*p != '\0' && p < ep)
 			p++;
 		if (p >= ep) {
-			error("unterminated generic string '%*s'", p - s, s);
+			error("unterminated generic string '%*s'", (int)(p - s), s);
 			break;
 		}
 		if (p++ == s)		/* empty string? */
@@ -904,13 +904,17 @@ static int loadobj(const char *objname, int ignore_suffix)
 				 * It might still want to export symbols,
 				 * although strictly speaking it shouldn't...
 				 * (Seen in pcmcia)
+				 * Architectures with function descriptors must
+				 * not do this, it is unsafe.
 				 */
+#ifndef HAS_FUNCTION_DESCRIPTORS
 				if (ELFW(ST_BIND)(objsym->info) == STB_GLOBAL) {
 					defsym[n_defsym++] = addsym(objsym->name,
 								    mod,
 								    SYM_DEFINED,
 								    ignore_suffix);
 				}
+#endif
 				continue;
 			}
 			/* else */
@@ -1060,12 +1064,9 @@ static int addksyms(char *file_syms)
 		if (!isspace(*line))	/* Adressless symbol? */
 			p = strtok(NULL, " \t\n");
 		/* The second word is either the symbol name or a type */
-		if (p && strlen(p) == 1) { /* System.map */
+		if (p && p[0] && !p[1]) { /* System.map */
 			is_mapfile = 1;
-			if (*p != '?')
-				p = NULL;
-			else
-				p = strtok(NULL, " \t\n");
+			p = strtok(NULL, " \t\n");
 		} else { /* /proc/ksyms copy */
 			if (p && strtok(NULL, " \t\n"))
 				p = NULL;
@@ -1083,7 +1084,7 @@ static int addksyms(char *file_syms)
 			if (!isspace(*line))	/* Adressless symbol? */
 				p = strtok(NULL, " \t\n");
 			if (is_mapfile) {
-				if (*p != '?')
+				if (!p || !p[0] || p[1])
 					continue;
 				p = strtok(NULL, " \t\n");
 				/* Sparc has symbols like '.div' that need to be
@@ -1258,6 +1259,13 @@ static int prtdepend(char *base_dir, int nflag)
 				if (m == nbdepmod)
 					tbdep[nbdepmod++] = mod;
 			} else {
+				/* Kludge to preserve old depmod behaviour without -u.
+				 * 2.4.13 change to keep going had the unwanted side
+				 * effect of always treating unresolved symbols as an
+				 * error.  Use the error() routine but do not count
+				 * any errors.  Remove in 2.5.
+				 */
+				int save_errors = errors;
 				if (!quiet && nberr == 0)
 					error("*** Unresolved symbols in %s",
 					      ptmod->name);
@@ -1266,6 +1274,7 @@ static int prtdepend(char *base_dir, int nflag)
 				nberr++;
 				if (flag_unresolved_error)
 					ret = 1;
+				errors = save_errors;
 			}
 		}
 		verbose("%s\n", ptmod->name + skipchars);
