@@ -265,6 +265,24 @@ obj_ia64_ins_imm64(Elf64_Xword v, Elf64_Addr *bundle, Elf64_Xword slot)
     return obj_reloc_ok;
 }
 
+/*
+ * add a immediate 60 value to the instruction at slot in bundle
+ */
+enum obj_reloc
+obj_ia64_ins_pcrel60b(Elf64_Xword v, Elf64_Addr *bundle, Elf64_Xword slot)
+{
+    Elf64_Xword ins;
+
+    assert(slot == 2);
+
+    ins = obj_ia64_ins_extract_from_bundle(bundle, slot);
+    ins &= 0xffffffee00001fff;
+    ins |= ((v & 0x8000000000000000) >> 27) | ((v & 0x0000000000fffff0) << 9);
+    obj_ia64_ins_insert_in_bundle(bundle, slot, ins);
+    obj_ia64_ins_insert_in_bundle(bundle, --slot, ((v & 0x7fffffffff000000) >> 22));
+    return obj_reloc_ok;
+}
+
 /*
  * create a plt entry
  */
@@ -852,6 +870,10 @@ arch_apply_relocation(struct obj_file *f,
 	}
 	break;
 
+    case R_IA64_PCREL60B :    /* @pcrel(sym + add), brl */
+	ret = obj_ia64_ins_pcrel60b(v - dot, bundle, slot);
+	break;
+
     case R_IA64_PCREL32LSB :    /* @pcrel(sym + add), data4 LSB */
 	COPY_32LSB(loc, v-dot);
 	break;
